; TABS = 4 spaces

;Port D:	LCD output for higher nibble (7-4)
;Port C:	LCD output for lower nibble  (3-0)
;Port B:	input to display as hexadecimal
;Port C6:	RESET (not used)
;Port D6:	common for both LCD digits

;The reset pin (PC6/AD6/PCINT14) is not used, so the chip can be reprogrammed for other uses.

		.include "iotiny48.inc"
		.include "macros.inc"

		;initialize stack pointer
		ldi		r16,0xDF
		out		SPL,r16

		;set Data Direction Registers as outputs
		ldi		r16,0xFF
		out		DDRD,r16
		out		DDRC,r16
		
		;main loop
		;The main loop is very simple. To display something on the LCD, we have to switch
		;the voltage polarity at a frequency of about 100Hz

		;OFF segment bit has the same bit value as Common
		;ON  segment bit has the inverted value of Common
1:
		;For a half cycle, display the complement of both digits
		
		in		r1,PINB		;read value to display from port B
		mov		r16,r1		;load byte to display
		
		andi	r16,0x0f	;keep lower nibble (bits 3-0)
		rcall	get7seg		;get corresponding 7-segment code for nibble
		com		r0			;invert outputs
		out		PORTC,r0	;put on display lines for LSD

		mov		r16,r1		;load byte to display
		andi	r16,0xf0	;keep higher nibble (bits 7-4)

		lsr		r16			;shift them into 3-0 to get the 7-segment code
		lsr		r16
		lsr		r16
		lsr		r16
		
		rcall	get7seg		;get corresponding 7-segment code for nibble
		com		r0			;invert outputs
		out		PORTD,r0	;put on display lines for MSD and common for both digits
		
		rcall	delay		;pause for about 5ms

		;For a half cycle, display both digits
		
		mov		r16,r1		;load byte to display

		andi	r16,0x0f	;keep lower nibble (bits 3-0)
		rcall	get7seg		;get corresponding 7-segment code for nibble
		out		PORTC,r0	;put on display lines

		mov		r16,r1		;load byte to display
		andi	r16,0xf0	;keep higher nibble (bits 7-4)

		lsr		r16			;shift them into 3-0 to get the 7-segment code
		lsr		r16
		lsr		r16
		lsr		r16
		
		rcall	get7seg		;get corresponding 7-segment code for nibble
		out		PORTD,r0	;put on display lines for MSD and common for both digits
		
		rcall	delay		;pause for about 5ms

		rjmp	1b			;jump back to local label 1

;=================================================================
; 0		a
; 1		b
; 2		c
; 3		d
; 4		e
; 5		f
; 6		PORTC6 is used for RESET/PORTD6 is used to drive the common
; 7		g
tabseg: .byte 0x3f,0x06,0x9b,0x8f,0xa6,0xad,0xbd,0x07,0xbf,0xaf,0xb7,0xbc,0x39,0x9e,0xb9,0xb1

;Converts the binary value in bits 3-0 to a 7-segment code to be displayed.
get7seg:
		LDZIMM tabseg		;beginging of table in Z register

		add		r30,r16		;offset
		lpm					;read back value
		ret

delay:
		LD_R26_IMM 320		;Initial value
1:		sbiw	r26,1		;decrement R26
		brne	1b			;jump back to local label 1 if not equal zero
		ret
